/*
** Module   : ColorSpec.java
** Abstract : Implements Progress color-phrase details.
**
** Copyright (c) 2005-2012, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 NVS 20051026 NEW   @23274 Created initial version.
** 002 SIY 20080331 CHG   @37736 Improved support for COLOR_VALUE mode.
**                               Ref: #37732
** 003 NVS 20080420 CHG   @38047 Color value that encodes one of the regular
**                               color specifications MESSAGE/INPUT/NORMAL
**                               is not stored as value but converts into the
**                               corresponding specification right away.
** 004 SIY 20080429 CHG   @38143 Return invalid color if it is not present in
**                               color map.
** 005 SIY 20101122              Minor cleanups.
** 006 SIY 20101223              Complete rewrite due to change of color
**                               handling approach.
** 007 CS  20121031              Removed client specific references
** 008 HC  20140119              Implemented runtime support for the COLOR 
**                               protermcap-attribute.
*/

package com.goldencode.p2j.ui;

import java.io.Serializable;
import java.util.*;

import org.apache.commons.logging.*;

import com.goldencode.p2j.util.character;

/**
 * Class ColorSpec supports the color-phrase specification.
 * <ul>
 *   <li>- defines color-phrase instance
 *   <li>- NORMAL | INPUT | MESSAGES
 *   <li>- protermcap-attribute
 *   <li>- dos-hex-attribute
 * </ul>
 * The following options are NOT SUPPORTED:
 * <ul>
 *   <li>- VALUE(expression)
 *   <li>- BLINK- BRIGHT- fgnd-color bgnd-color (WINDOWS only)
 *   <li>- BLINK- RVV- UNDERLINE- BRIGHT- fgnd-color (WINDOWS only).
 * </ul>
 * <p>
 * This class does not interpret the meaning of the color value.
 *
 * @author   NVS
 * @version  2.0
 */
public class ColorSpec
implements Serializable
{
   private static final Log LOG = LogFactory.getLog(ColorSpec.class);
   
   /** COLOR_VALUE definition */
   public static final int COLOR_VALUE = 3;

   /** Color name constant for NORMAL color. */
   public static final String COLOR_NORMAL   = "NORMAL";

   /** Color name constant for INPUT color. */
   public static final String COLOR_INPUT    = "INPUT";

   /** Color name constant for MESSAGES color. */
   public static final String COLOR_MESSAGES = "MESSAGES";

   /** Color name constant for BOLD color. */
   public static final String COLOR_BOLD     = "BOLD";

   /** Color name constant for BLINK color. */
   public static final String COLOR_BLINK    = "BLINK";

   /** Color name definition value */
   private String colorValue = COLOR_NORMAL;

   /** Color mapping table common to all terminals */
   private static Map<String, Color> globalColorMap = new LinkedHashMap<String, Color>();
   
   /** Map of terminal color mappings. */
   private static Map<String, Map<String, Color>> terminalColorMap;

   /* Initialize local map */
   static
   {
      // the default mappings, the keys must be uppercase!
      globalColorMap.put(COLOR_NORMAL  , Color.DEFAULT);
      globalColorMap.put(COLOR_INPUT   , Color.DEFAULT.withUnderline());
      globalColorMap.put(COLOR_MESSAGES, Color.DEFAULT.withReverse());
      globalColorMap.put(COLOR_BOLD    , Color.DEFAULT.withBold());
      globalColorMap.put(COLOR_BLINK   , Color.DEFAULT.withBlink());
      
      // load the color map
      ColorMapLoader loader = new ColorMapLoader();
         
      Map<String, Map<String, Color>> loadedMap = loader.loadAllMappings();
      if (loadedMap != null)
      {
         // fill the terminalColorMap field, don't use the loaded map directly, we need to
         // normalize the keys
         Map<String, Map<String, Color>> localTerminalMap = new LinkedHashMap<String, Map<String, Color>>();
         for (Map.Entry<String, Map<String, Color>> termEntry : loadedMap.entrySet())
         {
            Map<String, Color> localColorMap = new LinkedHashMap<String, Color>();
            for (Map.Entry<String, Color> colorEntry : termEntry.getValue().entrySet())
            {
               localColorMap.put(colorEntry.getKey().trim().toUpperCase(), colorEntry.getValue());
            }
            localTerminalMap.put(termEntry.getKey().trim().toUpperCase(), termEntry.getValue());
         }
         terminalColorMap = localTerminalMap;
      } else
      {
         LOG.warn("Failed to initialize the terminal color map!");
      }
   }

   /**
    * Default constructor.
    */
   public ColorSpec()
   {
   }

   /**
    * Copy constructor.
    *
    * @param   color
    *          existing color specification to duplicate
    */
   public ColorSpec(ColorSpec color)
   {
      this.colorValue = color.colorValue;
   }

   /**
    * Main constructor.
    *
    * @param   value
    *          color name.
    */
   public ColorSpec(String value)
   {
      this.colorValue = value;
   }

   /**
    * Build color specification using integer value as name.
    *
    * @param   intValue
    *          Color number.
    */
   public ColorSpec(int intValue)
   {
      this(String.valueOf(intValue));
   }

   /**
    * Convert color specification into real color.
    *
    * @return  color value
    */
   public Color convert()
   {
      String normalizedColorValue = colorValue.trim().toUpperCase();
      
      // try the global static map first
      Color result = globalColorMap.get(normalizedColorValue);
      if (result != null)
      {
         return result;
      }
      
      // now the colors from the terminal specific color map
      character terminalType = LogicalTerminal.getTerminalType();
      if (terminalType != null && !terminalType.isUnknown())
      {
         Map<String, Color> terminalColors = terminalColorMap.get(terminalType.getValue().toUpperCase());
         if (terminalColors != null)
         {
            result = terminalColors.get(normalizedColorValue);
            if (result != null)
            {
               return result;
            }
         }
      }
      
      // fall back to the NORMAL color
      return globalColorMap.get(COLOR_NORMAL);
   }
}
