/*
** Module   : SceenDriver.java
** Abstract : Common interface for all output drivers.
**
** Copyright (c) 2010-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 SIY 20100629 Created initial version
** 002 SIY 20100804 Changes in imports, some refactorings.
** 003 SIY 20101202 Method getAcsChar() replaced with getGraphics().
** 004 SIY 20110112 Changes related to new color handling scheme.
** 005 SIY 20110927 Updated javadoc.
** 006 GES 20111005 Updated import to match refactoring.
** 007 SIY 20111202 Added explicit import for List.
** 008 SIY 20111221 Added widget factory accessor.
** 009 EVL 20130725 Removed CHARVA package, using javadoc KeyProcessor from console driver.
** 010 CA  20131004 Added API to identify the default window-system used by a certain driver.
** 011 EVL 20131212 Added method to check if the driver is in batch mode.
** 012 HC  20140119 Implemented runtime support for the COLOR 
**                  protermcap-attribute.
*/

package com.goldencode.p2j.ui.client.driver;

import com.goldencode.p2j.main.ClientParameters;
import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.chui.driver.*;
import com.goldencode.p2j.ui.client.chui.driver.console.*;

/**
 * Common interface for all screen drivers.
 * <p>
 * This interface contains all implementation-specific methods which should be 
 * implemented from scratch for each particular driver implementation. The part
 * of the driver which is more hardware independent is listed in 
 * {@link OutputPrimitives} interface and {@link DriverPrimitives} abstract 
 * class.
 * <p>
 * General requirements to the implementation:<br>
 * At present only character based UI is supported. Therefore, all interaction
 * with the driver assumes existence of some kind of character-based terminal. 
 * All coordinates passed are coordinates of the character row and column (Y and 
 * X respectively). All coordinates are 0-based and Y axis has down direction. 
 * Upper left corner has coordinates (0,0) and lower right -  
 * (@link OutputPrimitives#screenWidth()} - 1, 
 * {{@link OutputPrimitives#screenHeight()} - 1). There is no limitations on 
 * actual terminal size, but at present dynamic screen size change is not 
 * supported - once application starts it assumes that terminal has fixes size 
 * which remains stable until application if finished. Particular implementation
 * may return terminal screen size different than phisical screen size, but
 * in this case it is responsible for handling of necessary 
 * scrolling/panning/clipping, because P2J has no support for this funtionality
 * and assumes that entire reported by driver screen area is visible to user.
 * Note that requirement for fixed size of terminal screen applies even if 
 * terminal is suspended and then resumed. Handling size changes (if necessary)
 * is up to the driver implementation.
 */
public interface ScreenDriver
{
   /**
    * Set cursor status (ON/OFF).
    * <p>
    * The implementation of this method should show/hide actual screen 
    * cursor visible to user. Remaining code assumes that actual cursor status
    * visible to user is properly changed upon exit from this method.
    * 
    * @param   on
    *          <code>true</code> - show cursor, <code>false</code> - hide cursor.
    * @return  <code>true</code> if operation was successful.
    * 
    * @throws  IllegalStateException
    */
   public boolean setCursorStatus(boolean on) throws IllegalStateException;

   /**
    * Clear screen.
    * <p>
    * This method should clear the screen and perform necessary flushing of the 
    * system-specific buffers if necessary. Users of this method assume that 
    * actual screen appearance as it is visible to the user is properly changed
    * upon exit from this method.
    */
   public void clear();

   /**
    * Provide an audible feedback to user by generating a beep signal.
    * <p>
    * There is no specific requirements to this method and actual implementation
    * may do nothing if there is no way to perform audible notification to the 
    * user.
    */
   public void beep();

   /**
    * Check if driver supports color output.
    * <p>
    * This method may be used to check if color output is supported by 
    * particular driver implementation.
    * <p>
    * TODO: at present there is no references to this method, can it be dropped
    * completely?
    * 
    * @return  <code>true</code> if colors are supported.
    */
   public boolean hasColors();
   
   /**
    * Checks if the current driver is running in batch mode.
    * 
    * @return  <code>true</code> for batch mode, <code>false</code> otherwise.
    * 
    */
   public boolean inBatchMode(); 

   /**
    * Read single keystroke from keyboard. Returned code should be either proper
    * Progress 4GL/P2J key code or -1 if environment-specific input cannot be 
    * properly translated.
    * <p>
    * The list of codes and detailed description is provided in {@link Keyboard}
    * class. The {@link KeyProcessor} can serve as an example of the key 
    * translation implementation.
    * 
    * @return  key code or -1 if no keys were pressed since last call.
    */
   public int readKey();

   /**
    * Reset current screen mode.
    * <p>
    * Upon exit from this method P2J assumes that screen is in correct state
    * and can be used for interaction with the user.
    */
   public void resetMode();

   /**
    * Suspend all screen I/O operations. If implementation is terminal-based, 
    * then terminal should be released to allow other application executed 
    * from P2J access terminal.
    */
   public void suspend();

   /**
    * Resume normal driver operation. If implementation is terminal-based,
    * they terminal should be acquired again and driver should be ready to 
    * perform regular screen I/O operations.
    */
   public void resume();

   /**
    * Obtain system/implementation specific terminal type string.
    * <p>
    * At present this is used to implement 4GL TERMINAL function and do some 
    * terminal-specific workarounds. At present workarounds exist for "vt320" 
    * terminal type. If implementation requires no special workarounds, then 
    * it should avoid returning "vt320" as terminal type string.  
    * 
    * @return  terminal type string.
    */
   public String terminalType();

   /**
    * Change terminal type to specified one. The terminal-based implementation
    * should try to change terminal type to specified one. It is assumed that 
    * passed terminal string is specific to particular driver implementation 
    * and recognized by it. 
    * 
    * @param   name
    */
   public void setTerminalType(String name);

   /**
    * Get driver-specific direct output primitives implementation.
    *
    * @return  driver-specific instance of direct screen output primitives.
    */
   public OutputPrimitives getPrimitives();

   /**
    * Get driver-specific widget factory.
    * 
    * @return  reference to widget factory.
    */
   public WidgetFactory getFactory();
   
   /**
    * Driver initialization.
    */
   public void init();

   /**
    * Get the default WINDOW-SYSTEM used by this driver. See {@link ClientParameters} for the list
    * of possible values.
    * 
    * @return   See above.
    */
   public String getWindowSystem();
}
